@@ -3,6 +3,7 @@
 #include <stdio.h>
 #include <string.h>
 #include <strings.h>
+#include <sys/utsname.h>
 
 #include <debian-installer/system/subarch.h>
 
@@ -11,6 +12,13 @@ struct map {
 	char *ret;
 };
 
+static const char *supported_generic_subarches[] = {
+    "dove",
+    "omap",
+    "omap4",
+    NULL
+};
+
 static struct map map_hardware[] = {
     { "Acorn-RiscPC" , "rpc" },
     { "EBSA285" , "netwinder" },
@@ -18,8 +26,22 @@ static struct map map_hardware[] = {
     { "Chalice-CATS" , "netwinder" },
     { "co-EBSA285" , "netwinder" },
     { "Compaq-PersonalServer" , "netwinder" },
+    { "Freescale MX51 Babbage Board", "imx51" }, /* iMX51 reference hardware. */
     { "ADS" , "ads" }, /* Matches only ADS boards. Put any exceptions before. */
     { "Applied Data Systems" , "ads" }, /* More ADS boards. */
+    { "HP t5325 Thin Client", "kirkwood" },
+    { "Marvell DB-88F6281-BP Development Board", "kirkwood" },
+    { "Marvell RD-88F6192-NAS Development Board", "kirkwood" },
+    { "Marvell RD-88F6281 Reference Board", "kirkwood" },
+    { "Marvell GuruPlug Reference Board", "kirkwood" },
+    { "Marvell OpenRD Base Board", "kirkwood" },
+    { "Marvell OpenRD Client Board", "kirkwood" },
+    { "Marvell OpenRD Ultimate Board", "kirkwood" },
+    { "Marvell SheevaPlug Reference Board", "kirkwood" },
+    { "Marvell eSATA SheevaPlug Reference Board", "kirkwood" },
+    { "QNAP TS-119/TS-219", "kirkwood" },
+    { "QNAP TS-41x", "kirkwood" },
+    { "Seagate FreeAgent DockStar", "kirkwood" },
     { "Buffalo/Revogear Kurobox Pro", "orion5x" },
     { "D-Link DNS-323", "orion5x" },
     { "QNAP TS-109/TS-209", "orion5x" },
@@ -34,6 +56,7 @@ static struct map map_hardware[] = {
     { "Thecus N2100", "iop32x" },
     { "Thecus N4100", "iop32x" },
     { "GLAN Tank", "iop32x" },
+    { "Lanner EM7210", "iop32x" },
     { "Intel IQ80331", "iop33x" },
     { "Intel IQ80332", "iop33x" },
     { "ADI Engineering Coyote", "ixp4xx" },
@@ -90,3 +113,27 @@ const char *di_system_subarch_analyze(void)
 
 	return "unknown";
 }
+
+const char *di_system_subarch_analyze_guess(void)
+{
+	struct utsname sysinfo;
+	size_t uname_release_len, i;
+
+	/* Attempt to determine subarch based on kernel release version */
+	uname(&sysinfo);
+	uname_release_len = strlen(sysinfo.release);
+
+	for (i = 0; supported_generic_subarches[i] != NULL; i++)
+	{
+		size_t subarch_len = strlen (supported_generic_subarches[i]);
+		if (!strncmp(sysinfo.release+uname_release_len-subarch_len,
+			supported_generic_subarches[i],
+			subarch_len))
+		{
+			return supported_generic_subarches[i];
+		}
+	}
+
+	/* If we get here, try falling back on the normal detection method */
+	return di_system_subarch_analyze();
+}
